<?xml version="1.0"?>
<!--                                                                        -->
<!-- Licensed Materials - Property of IBM                                   -->
<!-- (C) Copyright IBM Corp. 2003, 2004.  All Rights Reserved.              -->
<!--                                                                        -->
<!-- US Government Users Restricted Rights - Use, duplication or disclosure -->
<!-- restricted by GSA ADP Schedule Contract with IBM Corp.                 -->
<!--                                                                        -->
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
	xmlns:xmi="http://www.omg.org/XMI"
    xmlns:uml="http://www.eclipse.org/uml2/1.0.0/UML"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:publish="http://www.ibm.com/Rational/XTools/Publish"
    xmlns:xalan="http://xml.apache.org/xalan"
    exclude-result-prefixes="xmi uml xsi publish xalan">
    
    <!-- =========================== Imports ============================================== -->
    <xsl:import href="StringUtility.xsl"/>
    
    
    <xsl:output method="html"/>
    <!-- The template outputHTMLHeader will create a header element for HTML.  This will include a 
  	   title element plus description, keyword and generator meta tags.  For the title, description 
  	   and keywords strings the template will use the replaceSubString template to replace a token 
  	   with a replace string in a specified pattern before using the string in the header.
       
       Parameters:
         titlePattern            - The pattern to use as the title (e.g. "Contents for #name")
         titleReplaceToken       - The token to replace in the title pattern (e.g. "#name").
         titleReplaceWith        - The string to replace any titleReplaceTokens found (e.g. 
                                   select="@name").
         descriptionPattern      - The pattern to use as the description meta tag (e.g. "This is 
                                   the informaiton for the element #name").
         descriptionReplaceToken - The token to replace in the description pattern (e.g. "#name").
         descriptionReplaceWith  - The string to replace any descriptionReplaceTokens found (e.g. 
                                   select="@name")
         keywordsPattern         - The pattern to use as the keyword meta tag (e.g. "#name")
         keywordsReplaceToken    - The token to replace in the keyword pattern (e.g. "#name")
         keywordsReplaceWith     - The string to replace any keywordReplaceTokens found (e.g. 
                                   select="@name")
         generator               - The string to place in the "generator" meta tag.
  -->
    <xsl:template name="outputHTMLHeader">
        <xsl:param name="titlePattern" select="@name"/>
        <xsl:param name="titleReplaceToken">
            <xsl:text>#name</xsl:text>
        </xsl:param>
        <xsl:param name="titleReplaceWith" select="@name"/>
        <xsl:param name="descriptionPattern" select="''"/>
        <xsl:param name="descriptionReplaceToken">
            <xsl:text>#name</xsl:text>
        </xsl:param>
        <xsl:param name="descriptionReplaceWith" select="@name"/>
        <xsl:param name="keywordsPattern" select="''"/>
        <xsl:param name="keywordsReplaceToken">
            <xsl:text>#name</xsl:text>
        </xsl:param>
        <xsl:param name="keywordsReplaceWith" select="@name"/>
        <xsl:param name="csslink" select="WebPublish.css"/>
        <xsl:variable name="title">
            <xsl:call-template name="replaceSubString">
                <xsl:with-param name="theString">
                    <xsl:value-of select="$titlePattern"/>
                </xsl:with-param>
                <xsl:with-param name="matchSubString">
                    <xsl:value-of select="$titleReplaceToken"/>
                </xsl:with-param>
                <xsl:with-param name="replaceWith">
                    <xsl:value-of select="$titleReplaceWith"/>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="description">
            <xsl:call-template name="replaceSubString">
                <xsl:with-param name="theString">
                    <xsl:value-of select="$descriptionPattern"/>
                </xsl:with-param>
                <xsl:with-param name="matchSubString">
                    <xsl:value-of select="$descriptionReplaceToken"/>
                </xsl:with-param>
                <xsl:with-param name="replaceWith">
                    <xsl:value-of select="$descriptionReplaceWith"/>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="keywords">
            <xsl:call-template name="replaceSubString">
                <xsl:with-param name="theString">
                    <xsl:value-of select="$keywordsPattern"/>
                </xsl:with-param>
                <xsl:with-param name="matchSubString">
                    <xsl:value-of select="$keywordsReplaceToken"/>
                </xsl:with-param>
                <xsl:with-param name="replaceWith">
                    <xsl:value-of select="$keywordsReplaceWith"/>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <head>
            <title>
                <xsl:value-of select="$title"/>
            </title>
            <xsl:if test="$globalGeneratorName != ''">
                <meta>
                    <xsl:attribute name="name">generator</xsl:attribute>
                    <xsl:attribute name="content">
                        <xsl:value-of select="$globalGeneratorName"/>
                    </xsl:attribute>
                </meta>
            </xsl:if>
            <xsl:if test="$description != ''">
                <meta>
                    <xsl:attribute name="name">description</xsl:attribute>
                    <xsl:attribute name="content">
                        <xsl:value-of select="$description"/>
                    </xsl:attribute>
                </meta>
            </xsl:if>
            <xsl:if test="$keywords != ''">
                <meta>
                    <xsl:attribute name="keywords">
                        <xsl:value-of select="$keywords"/>
                    </xsl:attribute>
                </meta>
            </xsl:if>
            <link rel="stylesheet" type="text/css" href="WebPublish.css"/>
        </head>
    </xsl:template>
    <!-- Generate the HTML for a combined image and text link.  It creates both an image and a text 
       link pointing to the same href with a single space between them.  The image is optional so 
       this template can also generate normal links too.
    
       Parameters:
         link   - The value for href of both links.
         image  - The image to display before the text.
         text   - The text to display next to the image.
         target - The optional value for the target attribute.
  -->
    <xsl:template name="outputImageAndTextLinkHTML">
        <xsl:param name="link"/>
        <xsl:param name="image" select="''"/>
        <xsl:param name="text" select="@name"/>
        <xsl:param name="target" select="''"/>
			 
		<xsl:variable name="label">
			<xsl:if test="$text != ''">
				<xsl:value-of select="$text"/>
			</xsl:if>
			<xsl:if test="$text = ''">
				<xsl:text>Unnamed</xsl:text>
			</xsl:if>
        </xsl:variable>
        
        <!-- Create a link using the image -->
        <xsl:if test="$image != ''">
            <a>
                <xsl:if test="$target != ''">
                    <xsl:attribute name="target">
                        <xsl:value-of select="$target"/>
                    </xsl:attribute>
                </xsl:if>
                <xsl:attribute name="href">
                    <xsl:value-of select="$link"/>
                </xsl:attribute>
                <img border="0">
                    <xsl:attribute name="src">
                        <xsl:value-of select="$image"/>
                    </xsl:attribute>
                    <xsl:attribute name="alt">
                        <xsl:value-of select="$label"/>
                    </xsl:attribute>
                </img>
            </a>
            <xsl:text>&#160;</xsl:text>
        </xsl:if>
        <!-- Create a link using the text -->
        <a>
            <xsl:if test="$target != ''">
                <xsl:attribute name="target">
                    <xsl:value-of select="$target"/>
                </xsl:attribute>
            </xsl:if>
            <xsl:attribute name="href">
                <xsl:value-of select="$link"/>
            </xsl:attribute>
            <xsl:value-of select="$label"/>
        </a>
    </xsl:template>
    
    <xsl:template name="createTableTitle">
		<xsl:param name="title"/>
		
		<tr>
            <td class="SummaryTitle" colspan="2">
                <a name="{$title}">
                    <xsl:copy-of select="$title"/>
                </a>
            </td>
        </tr>
    </xsl:template>
    
    <xsl:template name="createSummaryTableContentRow">
        <tr>
            <td>
                <xsl:call-template name="outputImageAndTextLinkHTML">
				<xsl:with-param name="link">
				<xsl:text>#linkID</xsl:text>
				<xsl:value-of select="@xmi:id"/>
				</xsl:with-param>
                <xsl:with-param name="image">
                <xsl:if test="@publish:icon != ''">
                <xsl:call-template name="createElementIconFileName"/>
                </xsl:if>
                </xsl:with-param>
                <xsl:with-param name="text" select="@name"/>
                </xsl:call-template>
                <!-- comments are processed when anchor element is traversed
                <xsl:for-each select="ownedComment">
                <xsl:value-of select="@body"/>
                </xsl:for-each>
                -->
            </td>
        </tr>
    </xsl:template>
    
    <xsl:template name="createDetailTableContentRow">
		<br/>
		<span class="ListItem">
			<xsl:if test="@publish:icon != ''">
				<!-- Put the element image next the title -->
				<img>
					<xsl:attribute name="src">
					<xsl:call-template name="createElementIconFileName"/>
					</xsl:attribute>
				</img>
				<xsl:text>&#160;</xsl:text>
			</xsl:if>
			<a name="linkID{@xmi:id}">
				<xsl:if test="@name != ''">
					<xsl:value-of select="@name"/>
				</xsl:if>
				<xsl:if test="@name = ''">
					<xsl:text>Unnamed</xsl:text>
				</xsl:if>
			</a>
		</span>
		<dl>
            <!-- comments are processed when anchor element is traversed
			<dd>
				<xsl:for-each select="ownedComment">
					<p><xsl:value-of select="@body"/></p>
				</xsl:for-each>
			</dd>
			-->
			<dt>
				<b><xsl:copy-of select="$propertiesTitle"/></b>
			</dt>
			<dd>
				<!-- Output the properties table -->
				<xsl:apply-templates select="publish:properties">
					<xsl:with-param name="indentCount" select="1"/>
					<xsl:with-param name="includeTitle" select="false()"/>
				</xsl:apply-templates>
			</dd>
		</dl>
    </xsl:template>
    
                                
    <xsl:template name="createSummaryTableContentRowForTemplateParamters">
		<tr>
		<!--
            <td class="OwnedMemberType">

				<xsl:call-template name="outputImageAndTextLinkHTML">
				<xsl:with-param name="link">
				<xsl:text>#linkID</xsl:text>
				<xsl:value-of select="@xmi:id"/>
				</xsl:with-param>
				<xsl:with-param name="image">
				<xsl:if test="@publish:icon != ''">
				<xsl:call-template name="createElementIconFileName"/>
				</xsl:if>
				</xsl:with-param>
				<xsl:with-param name="text" select="@name"/>
				</xsl:call-template>
            </td>
		-->
            
            <td>
				<xsl:for-each select="ownedParameteredElement">
					<xsl:call-template name="outputImageAndTextLinkHTML">
					<xsl:with-param name="link">
					<xsl:text>#linkID</xsl:text>
					<xsl:value-of select="@xmi:id"/>
					</xsl:with-param>
					<xsl:with-param name="image">
					<xsl:if test="@publish:icon != ''">
					<xsl:call-template name="createElementIconFileName"/>
					</xsl:if>
					</xsl:with-param>
					<xsl:with-param name="text" select="@name"/>
					</xsl:call-template>
				</xsl:for-each>
            
				&#160;
				
                <!-- comments are processed when anchor element is traversed
				<xsl:for-each select="ownedComment">
                <xsl:value-of select="@body"/>
                </xsl:for-each>
                -->
            </td>
        </tr>
    </xsl:template>
    
	<!-- used for Template INSTANCE summary tables  -->  
    <xsl:template name="createSummaryTableContentRowForTemplateInstanceParamters">
	<tr>
        <td>

			<xsl:for-each select="parameterSubstitution">
				<xsl:call-template name="outputImageAndTextLinkHTML">
				<xsl:with-param name="link">
				<xsl:text>#linkID</xsl:text>
				<xsl:value-of select="@xmi:id"/>
				</xsl:with-param>
				<xsl:with-param name="image">
				<xsl:if test="@publish:icon != ''">
				<xsl:call-template name="createElementIconFileName"/>
				</xsl:if>
				</xsl:with-param>
				<xsl:with-param name="text" select="@publish:qualifiedname"/>
				</xsl:call-template>
			</xsl:for-each>
			&#160;
            <!-- comments are processed when anchor element is traversed
			<xsl:for-each select="ownedComment">
            <xsl:value-of select="@body"/>
            </xsl:for-each>
            -->
        </td>
    </tr>
	</xsl:template>
	
<!-- -->
    <xsl:template name="createDetailTableForTemplateBinding">
		<br/>
		<span class="ListItem">
				<xsl:if test="@publish:icon != ''">
					<xsl:variable name="image">
						<xsl:call-template name="createElementIconFileName"/>
					</xsl:variable>				
					<img border="0">
						<xsl:attribute name="src">
							<xsl:value-of select="$image"/>
						</xsl:attribute>
						<xsl:attribute name="alt">
							<xsl:value-of select="@name"/>
						</xsl:attribute>
					</img>
					<xsl:text>&#160;</xsl:text>
				</xsl:if>
				
			<a name="templateBinding{@xmi:id}">
				<xsl:if test="@name != ''">
					<xsl:value-of select="@name"/>
				</xsl:if>
				<xsl:if test="@name = ''">
					<xsl:text>Unnamed</xsl:text>
				</xsl:if>
			</a>
		</span>
		<dl>
  			<dt>
				<b><xsl:copy-of select="$propertiesTitle"/></b>
			</dt>
			<dd>
				<!-- Output the properties table -->
				<xsl:apply-templates select="publish:properties">
					<xsl:with-param name="indentCount" select="1"/>
					<xsl:with-param name="includeTitle" select="false()"/>
				</xsl:apply-templates>
			</dd>
		</dl>
    </xsl:template>
    
	<!-- Template Parameter Substitution -->    
    <xsl:template name="createDetailTableForTemplateParameterSubstitutions">
		<br/>
		<xsl:for-each select="parameterSubstitution">
		<span class="ListItem">
			<xsl:if test="@publish:icon != ''">
					<xsl:variable name="image">
						<xsl:call-template name="createElementIconFileName"/>
					</xsl:variable>				
					<img border="0">
						<xsl:attribute name="src">
							<xsl:value-of select="$image"/>
						</xsl:attribute>
						<xsl:attribute name="alt">
							<xsl:value-of select="@name"/>
						</xsl:attribute>
					</img>
					<xsl:text>&#160;</xsl:text>
				</xsl:if>
				
			<a name="templateBinding{@xmi:id}">
				<xsl:if test="@name != ''">
					<xsl:value-of select="@name"/>
				</xsl:if>
				<xsl:if test="@name = ''">
					<xsl:text>Unnamed</xsl:text>
				</xsl:if>
			</a>
			
		</span>
		<dl>
  			<dt>
				<b><xsl:copy-of select="$propertiesTitle"/></b>
			</dt>
			<dd>
				<!-- Output the properties table -->
				<xsl:apply-templates select="publish:properties">
					<xsl:with-param name="indentCount" select="1"/>
					<xsl:with-param name="includeTitle" select="false()"/>
				</xsl:apply-templates>
			</dd>
		</dl>
		</xsl:for-each>
    </xsl:template>
    
    <xsl:template name="createDetailTableContentRowForTemplateParamters">
		<br/>
		<span class="ListItem">
			<!--
			<xsl:if test="@publish:icon != ''">
				<img>
					<xsl:attribute name="src">
					<xsl:call-template name="createElementIconFileName"/>
					</xsl:attribute>
				</img>
				<xsl:text>&#160;</xsl:text>
			</xsl:if>
			<a name="linkID{@xmi:id}">
				<xsl:if test="@name != ''">
					<xsl:value-of select="@name"/>
				</xsl:if>
				<xsl:if test="@name = ''">
					<xsl:text>Unnamed</xsl:text>
				</xsl:if>
			</a>
			
			&#160;:&#160;
			-->
			
			<xsl:for-each select="ownedParameteredElement">
			
				<xsl:if test="@publish:icon != ''">
					<xsl:variable name="image">
						<xsl:call-template name="createElementIconFileName"/>
					</xsl:variable>
				
					<img border="0">
						<xsl:attribute name="src">
							<xsl:value-of select="$image"/>
						</xsl:attribute>
						<xsl:attribute name="alt">
							<xsl:value-of select="@name"/>
						</xsl:attribute>
					</img>
					<xsl:text>&#160;</xsl:text>
				</xsl:if>
				
				<a name="linkID{@xmi:id}">
					<xsl:if test="@name != ''">
						<xsl:value-of select="@name"/>
					</xsl:if>
				</a>
				<!--
				<xsl:call-template name="outputImageAndTextLinkHTML">
				<xsl:with-param name="link">
				<xsl:text>#linkID</xsl:text>
				<xsl:value-of select="@xmi:id"/>
				</xsl:with-param>
				<xsl:with-param name="image">
				<xsl:if test="@publish:icon != ''">
				<xsl:call-template name="createElementIconFileName"/>
				</xsl:if>
				</xsl:with-param>
				<xsl:with-param name="text" select="@name"/>
				</xsl:call-template>
				-->
			</xsl:for-each>
		</span>
		<dl>
            <!-- comments are processed when anchor element is traversed
			<dd>
				<xsl:for-each select="ownedComment">
					<p><xsl:value-of select="@body"/></p>
				</xsl:for-each>
			</dd>
			-->
			<dt>
				<b><xsl:copy-of select="$propertiesTitle"/></b>
			</dt>
			<dd>
				<!-- Output the properties table -->
				<xsl:apply-templates select="publish:properties">
					<xsl:with-param name="indentCount" select="1"/>
					<xsl:with-param name="includeTitle" select="false()"/>
				</xsl:apply-templates>
			</dd>
		</dl>
    </xsl:template>
    
    <!-- useful utility for showing text with line breaks in it 
        Note that XML style end-of-line characters will not be displayed properly
        in the output HTML file unless you call this template
    -->
    <xsl:template name="replaceXML2HTMLLineBreaks">
        <xsl:param name="theText">ERROR-MISSING-PARAMETER</xsl:param>
        
        <!-- pass #1: replace DOS style end of line characters -->
        <xsl:variable name="replaceDosEols">
            <xsl:call-template name="replaceSubString">
                <xsl:with-param name="theString">
                    <xsl:value-of select="$theText"/>
                </xsl:with-param>
                <xsl:with-param name="matchSubString">
                    <xsl:text>&#xd;&#xa;</xsl:text>
                </xsl:with-param>
                <xsl:with-param name="replaceWith">
                    <xsl:text>&lt;/br&gt;</xsl:text>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        
        <!-- pass #2: replace UNIX style end of line characters -->
        <xsl:variable name="replaceUnixEols">
            <xsl:call-template name="replaceSubString">
                <xsl:with-param name="theString">
                    <xsl:value-of select="$replaceDosEols"/>
                </xsl:with-param>
                <xsl:with-param name="matchSubString">
                    <xsl:text>&#xa;</xsl:text>
                </xsl:with-param>
                <xsl:with-param name="replaceWith">
                    <xsl:text>&lt;/br&gt;</xsl:text>
                </xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
                
        <xsl:value-of select="$replaceUnixEols" disable-output-escaping="yes"/>
    </xsl:template>
</xsl:stylesheet>
